@@ -4,15 +4,17 @@
  * LiteSpeed Web Server Cache Manager
  *
  * @author LiteSpeed Technologies, Inc. (https://www.litespeedtech.com)
- * @copyright (c) 2017-2020
+ * @copyright (c) 2017-2021
  * ******************************************* */
 
 namespace Lsc\Wp\Panel;
 
-use \Lsc\Wp\Logger;
-use \Lsc\Wp\LSCMException;
-use \Lsc\Wp\Util;
-use \Lsc\Wp\WPInstall;
+use DirectoryIterator;
+use Exception;
+use Lsc\Wp\Logger;
+use Lsc\Wp\LSCMException;
+use Lsc\Wp\Util;
+use Lsc\Wp\WPInstall;
 
 abstract class ControlPanel
 {
@@ -34,7 +36,7 @@ abstract class ControlPanel
     /**
      * @var string
      */
-    const PANEL_API_VERSION = '1.13.2.1';
+    const PANEL_API_VERSION = '1.13.10.2';
 
     /**
      * @since 1.9
@@ -63,7 +65,7 @@ abstract class ControlPanel
     /**
      * @var int
      */
-    const PHP_TIMEOUT = 10;
+    const PHP_TIMEOUT = 30;
 
     /**
      * @var string
@@ -106,8 +108,8 @@ abstract class ControlPanel
     protected $apacheVHConf;
 
     /**
-     * @var null|mixed[][]  'docroots' => (index => docroots),
-     *                      'names' => (servername => index)
+     * @var null|array[]  'docroots' => (index => docroots),
+     *                    'names' => (servername => index)
      */
     protected $docRootMap = null;
 
@@ -146,10 +148,10 @@ abstract class ControlPanel
          * 'ob_gzhandler' etc.
          */
         $this->phpOptions = '-d disable_functions=ini_set -d opcache.enable=0 '
-                . '-d max_execution_time=' . static::PHP_TIMEOUT
-                . ' -d memory_limit=512M -d register_argc_argv=1 '
-                . '-d zlib.output_compression=0 -d output_handler= '
-                . '-d safe_mode=0 -d open_basedir=';
+            . '-d max_execution_time=' . static::PHP_TIMEOUT
+            . ' -d memory_limit=512M -d register_argc_argv=1 '
+            . '-d zlib.output_compression=0 -d output_handler= '
+            . '-d safe_mode=0 -d open_basedir=';
 
         $this->initConfPaths();
     }
@@ -164,7 +166,9 @@ abstract class ControlPanel
      * been set to a different class name than the one provided.
      *
      * @deprecated
-     * @param string  $className  A fully qualified control panel class name
+     *
+     * @param string $className  A fully qualified control panel class name.
+     *
      * @return ControlPanel|null  Object that extends ControlPanel abstract
      *                            class.
      * @throws LSCMException
@@ -176,13 +180,14 @@ abstract class ControlPanel
             if ( $className == 'custom' ) {
                 $lsws_home = realpath(__DIR__ . '/../../../../');
                 $customPanelFile =
-                    "{$lsws_home}/admin/lscdata/custom/CustomPanel.php";
+                    "$lsws_home/admin/lscdata/custom/CustomPanel.php";
 
                 if ( ! file_exists($customPanelFile)
                         || ! include_once $customPanelFile ) {
 
                     throw new LSCMException(
-                            "Unable to include file {$customPanelFile}");
+                        "Unable to include file $customPanelFile"
+                    );
                 }
 
                 $className = '\Lsc\Wp\Panel\CustomPanel';
@@ -193,18 +198,21 @@ abstract class ControlPanel
                 );
 
                 if ( ! $isSubClass ) {
-                    $msg = 'Class CustomPanel must extend class '
-                            . '\Lsc\Wp\Panel\CustomPanelBase';
-                    throw new LSCMException($msg);
+                    throw new LSCMException(
+                        'Class CustomPanel must extend class '
+                            . '\Lsc\Wp\Panel\CustomPanelBase'
+                    );
                 }
             }
 
             try{
                 static::$instance = new $className();
             }
-            catch ( \Exception $e ){
+            catch ( Exception $e ){
                 throw new LSCMException(
-                        "Could not create object with class name {$className}");
+                    "Could not create object with class name $className. "
+                        . "Error: {$e->getMessage()}"
+                );
             }
         }
         else {
@@ -212,8 +220,8 @@ abstract class ControlPanel
 
             if ( $instanceClassName != $className ) {
                 throw new LSCMException(
-                    "Could not initialize {$className} instance as an instance "
-                        . "of another class ({$instanceClassName}) has already "
+                    "Could not initialize $className instance as an instance "
+                        . "of another class ($instanceClassName) has already "
                         . 'been created.'
                 );
             }
@@ -227,7 +235,8 @@ abstract class ControlPanel
      *
      * @deprecated
      *
-     * @param string  $name
+     * @param string $name
+     *
      * @return ControlPanel  Object that extends ControlPanel abstract class.
      * @throws LSCMException
      */
@@ -261,8 +270,9 @@ abstract class ControlPanel
      * initialized if it has not yet been initialized already.
      *
      * @param string $className  Fully qualified class name.
+     *
      * @return ControlPanel  Object that extends ControlPanel abstract class.
-     * @throws LSCMException  Indirectly thrown by static::initByClassName().
+     * @throws LSCMException  Thrown directly and indirectly.
      */
     public static function getClassInstance( $className = '' )
     {
@@ -282,6 +292,7 @@ abstract class ControlPanel
      * Deprecated on 02/06/19. Use getClassInstance() instead.
      *
      * @deprecated
+     *
      * @return ControlPanel  Object that extends ControlPanel abstract class.
      */
     public static function getInstance()
@@ -291,8 +302,10 @@ abstract class ControlPanel
 
     /**
      *
-     * @param string  $serverName
+     * @param string $serverName
+     *
      * @return string|null
+     * @throws LSCMException  Thrown indirectly.
      */
     public function mapDocRoot( $serverName )
     {
@@ -336,11 +349,12 @@ abstract class ControlPanel
     public function verifyCacheSetup()
     {
         if ( !$this->isCacheEnabled() ) {
-            $msg = 'LSCache is not included in the current LiteSpeed license. '
+            throw new LSCMException(
+                'LSCache is not included in the current LiteSpeed license. '
                     . 'Please purchase the LSCache add-on or upgrade to a '
-                    . 'license type that includes LSCache and try again.';
-
-            throw new LSCMException($msg, LSCMException::E_PERMISSION);
+                    . 'license type that includes LSCache and try again.',
+                LSCMException::E_PERMISSION
+            );
         }
 
         $restartRequired = false;
@@ -362,17 +376,19 @@ abstract class ControlPanel
 
     /**
      *
-     * @param string  $vhCacheRoot
-     * @throws LSCMException  Indirectly thrown by $this->log() and
-     *                        $this->writeVHCacheRoot().
+     * @param string $vhCacheRoot
+     *
+     * @throws LSCMException  Thrown indirectly.
      */
     public function setVHCacheRoot( $vhCacheRoot = 'lscache' )
     {
         $this->log('Attempting to set VH cache root...', Logger::L_VERBOSE);
 
         if ( !file_exists($this->apacheVHConf) ) {
-            $this->createVHConfAndSetCacheRoot($this->apacheVHConf,
-                    $vhCacheRoot);
+            $this->createVHConfAndSetCacheRoot(
+                $this->apacheVHConf,
+                $vhCacheRoot
+            );
         }
         else {
             $this->writeVHCacheRoot($this->apacheVHConf, $vhCacheRoot);
@@ -381,7 +397,7 @@ abstract class ControlPanel
         $this->vhCacheRoot = $vhCacheRoot;
 
         $this->log(
-            "Virtual Host cache root set to {$vhCacheRoot}",
+            "Virtual Host cache root set to $vhCacheRoot",
             Logger::L_INFO
         );
 
@@ -391,7 +407,7 @@ abstract class ControlPanel
             /**
              * 01/29/19: Temporarily create top virtual host cache root
              * directory to avoid LSWS setting incorrect owner/group and
-             * permissions for the directory outside of the cage.
+             * permissions for the directory outside the cage.
              */
 
             $topVHCacheRoot = str_replace('/$vh_user', '', $vhCacheRoot);
@@ -411,22 +427,25 @@ abstract class ControlPanel
         $statusFile = '/tmp/lshttpd/.status';
 
         if ( !file_exists($statusFile) ) {
-            $msg = 'Cannot determine LSCache availability. Please start/switch '
-                    . 'to LiteSpeed Web Server before trying again.';
-
-            throw new LSCMException($msg, LSCMException::E_PERMISSION);
+            throw new LSCMException(
+                'Cannot determine LSCache availability. Please start/switch '
+                    . 'to LiteSpeed Web Server before trying again.',
+                LSCMException::E_PERMISSION
+            );
         }
 
         if ( ($f = fopen($statusFile, 'r')) === false ) {
-            throw new LSCMException('Cannot determine LSCache availability.',
-                    LSCMException::E_PERMISSION);
+            throw new LSCMException(
+                'Cannot determine LSCache availability.',
+                LSCMException::E_PERMISSION
+            );
         }
 
         fseek($f, -128, SEEK_END);
         $line = fread($f, 128);
         fclose($f);
 
-        if ( preg_match('/FEATURES: ([0-9\.]+)/', $line, $m)
+        if ( preg_match('/FEATURES: ([0-9.]+)/', $line, $m)
                 && ($m[1] & 1) == 1 ) {
 
             return true;
@@ -438,9 +457,11 @@ abstract class ControlPanel
     /**
      * return array of docroots, can set index from and batch
      *
-     * @param int       $offset
-     * @param null|int  $length
+     * @param int      $offset
+     * @param null|int $length
+     *
      * @return string[]
+     * @throws LSCMException  Thrown indirectly.
      */
     public function getDocRoots( $offset = 0, $length = null )
     {
@@ -454,11 +475,18 @@ abstract class ControlPanel
     /**
      * Used in PleskEscalate.
      *
-     * @return mixed[][]
+     * @return array[]
+     *
+     * @noinspection PhpUnused
+     * @noinspection PhpDocMissingThrowsInspection
      */
     public function getDocrootMap()
     {
         if ( $this->docRootMap == null ) {
+            /**
+             * LSCMException not thrown in Plesk implementation.
+             * @noinspection PhpUnhandledExceptionInspection
+             */
             $this->prepareDocrootMap();
         }
 
@@ -507,8 +535,19 @@ abstract class ControlPanel
      */
     abstract protected function initConfPaths();
 
+    /**
+     *
+     * @throws LSCMException  Thrown in some existing implementations.
+     */
     abstract protected function prepareDocrootMap();
 
+    /**
+     *
+     * @param WPInstall $wpInstall
+     *
+     * @return string
+     * @throws LSCMException  Thrown in some existing implementations.
+     */
     abstract public function getPhpBinary( WPInstall $wpInstall );
 
     /**
@@ -516,12 +555,13 @@ abstract class ControlPanel
      *
      * Note: Visibility is public to better accommodate escalation functions.
      *
-     * @param string  $confDir  Directory to be searched.
+     * @param string $confDir  Directory to be searched.
+     *
      * @return string
      */
     public function cacheRootSearch( $confDir )
     {
-        $files = new \DirectoryIterator($confDir);
+        $files = new DirectoryIterator($confDir);
 
         foreach ( $files as $file ) {
             $filename = $file->getFilename();
@@ -543,7 +583,8 @@ abstract class ControlPanel
     /**
      * Note: Visibility is public to better accommodate escalation functions.
      *
-     * @param string  $file
+     * @param string $file
+     *
      * @return string
      */
     public function getCacheRootSetting( $file )
@@ -575,9 +616,9 @@ abstract class ControlPanel
         if ( file_exists($serverConf) ) {
             $file_content = file_get_contents($serverConf);
 
-            if ( preg_match('!<cacheStorePath>(.+)</cacheStorePath>!i',
-                            $file_content, $matches) ) {
+            $pattern = '!<cacheStorePath>(.+)</cacheStorePath>!i';
 
+            if ( preg_match($pattern, $file_content, $matches) ) {
                 $svrCacheRoot = trim($matches[1]);
             }
         }
@@ -633,15 +674,14 @@ abstract class ControlPanel
 
     /**
      *
-     * @param string  $msg
-     * @param int     $level
-     * @throws LSCMException  Indirectly thrown by Logger::error(),
-     *                        Logger::warn(), Logger::notice(), Logger::info(),
-     *                        Logger::verbose(), and Logger::debug().
+     * @param string $msg
+     * @param int    $level
+     *
+     * @throws LSCMException  Thrown indirectly.
      */
     protected function log( $msg, $level )
     {
-        $msg = "{$this->panelName} - {$msg}";
+        $msg = "$this->panelName - $msg";
 
         switch ($level) {
 
@@ -675,7 +715,8 @@ abstract class ControlPanel
 
     /**
      *
-     * @param string  $svrCacheRoot
+     * @param string $svrCacheRoot
+     *
      * @throws LSCMException  Thrown directly and indirectly.
      */
     public function setServerCacheRoot( $svrCacheRoot = '' )
@@ -690,13 +731,13 @@ abstract class ControlPanel
         }
 
         $cacheRootLine =
-            "<IfModule LiteSpeed>\nCacheRoot {$cacheroot}\n</IfModule>\n\n";
+            "<IfModule LiteSpeed>\nCacheRoot $cacheroot\n</IfModule>\n\n";
 
         if ( !file_exists($this->apacheConf) ) {
             file_put_contents($this->apacheConf, $cacheRootLine);
             chmod($this->apacheConf, 0644);
 
-            $this->log("Created file {$this->apacheConf}", Logger::L_VERBOSE);
+            $this->log("Created file $this->apacheConf", Logger::L_VERBOSE);
         }
         else {
 
@@ -714,19 +755,21 @@ abstract class ControlPanel
             else {
                 $file_contents = file($this->apacheConf);
 
-                if ( preg_grep('/^\s*<IfModule +LiteSpeed *>/im', $file_contents) ) {
+                $pattern = '/^\s*<IfModule +LiteSpeed *>/im';
+
+                if ( preg_grep($pattern, $file_contents) ) {
 
                     if ( preg_grep('/^\s*CacheRoot +/im', $file_contents) ) {
                         $file_contents = preg_replace(
                             '/^\s*CacheRoot +.+/im',
-                            "CacheRoot {$cacheroot}",
+                            "CacheRoot $cacheroot",
                             $file_contents
                         );
                     }
                     else {
                         $file_contents = preg_replace(
                             '/^\s*<IfModule +LiteSpeed *>/im',
-                            "<IfModule LiteSpeed>\nCacheRoot {$cacheroot}",
+                            "<IfModule LiteSpeed>\nCacheRoot $cacheroot",
                             $file_contents
                         );
                     }
@@ -742,12 +785,12 @@ abstract class ControlPanel
         $this->serverCacheRoot = $cacheroot;
 
         $this->log(
-            "Server level cache root set to {$cacheroot}",
+            "Server level cache root set to $cacheroot",
             Logger::L_INFO
         );
 
         if ( file_exists($cacheroot) ) {
-            exec("/bin/rm -rf {$cacheroot}");
+            exec("/bin/rm -rf $cacheroot");
             $this->log(
                 'Server level cache root directory removed for proper '
                     . 'permission.',
@@ -758,29 +801,33 @@ abstract class ControlPanel
 
     /**
      *
-     * @param array   $file_contents
-     * @param string  $vhCacheRoot
+     * @param array  $file_contents
+     * @param string $vhCacheRoot
+     *
      * @return array
      */
-    abstract protected function addVHCacheRootSection( $file_contents,
-            $vhCacheRoot = 'lscache' );
+    abstract protected function addVHCacheRootSection(
+        $file_contents,
+        $vhCacheRoot = 'lscache'
+    );
 
     /**
      *
-     * @param string  $vhConf
+     * @param string $vhConf
+     *
      * @throws LSCMException  Thrown directly and indirectly.
      */
     public function writeVHCacheRoot( $vhConf, $vhCacheRoot = 'lscache' )
     {
         if ( !is_writable($vhConf) ) {
             throw new LSCMException(
-                "Could not write to VH config {$vhConf}. No changes made.",
+                "Could not write to VH config $vhConf. No changes made.",
                 LSCMException::E_PERMISSION
             );
         }
         if ( !Util::createBackup($vhConf) ) {
             throw new LSCMException(
-                "Could not backup Virtual Host config file {$vhConf}. No "
+                "Could not backup Virtual Host config file $vhConf. No "
                     . 'changes made.',
                 LSCMException::E_PERMISSION
             );
@@ -791,38 +838,45 @@ abstract class ControlPanel
         if ( preg_grep('/^\s*<IfModule +LiteSpeed *>/im', $file_contents) ) {
 
             if ( preg_grep('/^\s*CacheRoot +/im', $file_contents) ) {
-                $modified_contents = preg_replace('/^\s*CacheRoot +.+/im',
-                        "CacheRoot {$vhCacheRoot}", $file_contents);
+                $modified_contents = preg_replace(
+                    '/^\s*CacheRoot +.+/im',
+                    "CacheRoot $vhCacheRoot",
+                    $file_contents
+                );
             }
             else {
                 $modified_contents = preg_replace(
                     '/^\s*<IfModule +LiteSpeed *>/im',
-                    "<IfModule LiteSpeed>\nCacheRoot {$vhCacheRoot}",
+                    "<IfModule LiteSpeed>\nCacheRoot $vhCacheRoot",
                     $file_contents
                 );
             }
         }
         else {
             $modified_contents =
-                    $this->addVHCacheRootSection($file_contents, $vhCacheRoot);
+                $this->addVHCacheRootSection($file_contents, $vhCacheRoot);
         }
 
         if ( file_put_contents($vhConf, $modified_contents) === false ) {
-            throw new LSCMException("Failed to write to file {$vhConf}.",
-                    LSCMException::E_PERMISSION);
+            throw new LSCMException(
+                "Failed to write to file $vhConf.",
+                LSCMException::E_PERMISSION
+            );
         }
 
-        $this->log("Updated file {$vhConf}.", Logger::L_DEBUG);
+        $this->log("Updated file $vhConf.", Logger::L_DEBUG);
     }
 
     /**
      * Note: Visibility is public to better accommodate escalation functions.
      *
-     * @param string  $vhConf
-     * @param string  $vhCacheRoot
+     * @param string $vhConf
+     * @param string $vhCacheRoot
      */
-    abstract public function createVHConfAndSetCacheRoot( $vhConf,
-            $vhCacheRoot = 'lscache' );
+    abstract public function createVHConfAndSetCacheRoot(
+        $vhConf,
+        $vhCacheRoot = 'lscache'
+    );
 
     /**
      * Note: Visibility is public to better accommodate escalation functions.
@@ -894,9 +948,7 @@ abstract class ControlPanel
             static::populateMinAPIVerFile();
         }
 
-        $minVer = trim(file_get_contents($minVerFile));
-
-        return $minVer;
+        return trim(file_get_contents($minVerFile));
     }
 
     /**
@@ -922,13 +974,33 @@ abstract class ControlPanel
      *
      * @since 1.9
      *
-     * @param string  $panelAPIVer  Shared code API version used by the panel
-     *                              plugin.
+     * @param string $panelAPIVer  Shared code API version used by the panel
+     *                             plugin.
+     *
      * @return int
      */
     public static function checkPanelAPICompatibility( $panelAPIVer )
     {
-        $supportedAPIVers = array (
+        $supportedAPIVers = array(
+            '1.13.10.2',
+            '1.13.10.1',
+            '1.13.10',
+            '1.13.9',
+            '1.13.8',
+            '1.13.7.1',
+            '1.13.7',
+            '1.13.6',
+            '1.13.5.2',
+            '1.13.5.1',
+            '1.13.5',
+            '1.13.4.4',
+            '1.13.4.3',
+            '1.13.4.2',
+            '1.13.4.1',
+            '1.13.4',
+            '1.13.3.1',
+            '1.13.3',
+            '1.13.2.2',
             '1.13.2.1',
             '1.13.2',
             '1.13.1',
@@ -982,8 +1054,9 @@ abstract class ControlPanel
      *
      * @deprecated 1.9  Use checkPanelAPICompatibility() instead.
      *
-     * @param string  $panelAPIVer  Shared code API version used by the panel
-     *                               plugin.
+     * @param string $panelAPIVer  Shared code API version used by the panel
+     *                             plugin.
+     *
      * @return boolean
      */
     public static function isPanelAPICompatible( $panelAPIVer )
