@@ -4,13 +4,13 @@
  * LiteSpeed Web Server Cache Manager
  *
  * @author LiteSpeed Technologies, Inc. (https://www.litespeedtech.com)
- * @copyright (c) 2018-2020
+ * @copyright (c) 2018-2021
  * *******************************************
  */
 
 namespace Lsc\Wp;
 
-use \Lsc\Wp\Context\Context;
+use Lsc\Wp\Context\Context;
 
 /**
  * Calls WP internal functions in SHORTINIT mode.
@@ -31,6 +31,12 @@ class WPCaller
      */
     private $currInstall;
 
+    /**
+     * @since 1.13.4.4
+     * @var string
+     */
+    private $advancedCacheFile;
+
     /**
      * @var boolean
      */
@@ -59,21 +65,38 @@ class WPCaller
 
     /**
      *
-     * @param WPInstall  $curInstall
-     * @param boolean    $loadLscwp
+     * @param WPInstall $curInstall
+     * @param boolean   $loadLscwp
+     *
      * @throws LSCMException  Thrown indirectly.
      */
     private function __construct( WPInstall $curInstall, $loadLscwp )
     {
         $this->currInstall = $curInstall;
         $this->loadLscwp = $loadLscwp;
+
+        $this->init();
+    }
+
+    /**
+     *
+     * @since 1.13.4.4
+     *
+     * @throws LSCMException  Thrown indirectly.
+     */
+    private function init()
+    {
+        $this->advancedCacheFile =
+                "{$this->currInstall->getPath()}/wp-content/advanced-cache.php";
+
         $this->initWp();
     }
 
     /**
      *
-     * @param WPInstall  $currInstall
-     * @param boolean    $loadLscwp
+     * @param WPInstall $currInstall
+     * @param boolean   $loadLscwp
+     *
      * @return WPCaller
      * @throws LSCMException  Thrown indirectly.
      */
@@ -91,8 +114,9 @@ class WPCaller
 
     /**
      *
-     * @param int     $errno  Not used at this time.
-     * @param string  $errstr
+     * @param int    $errno   Not used at this time.
+     * @param string $errstr
+     *
      * @return boolean
      *
      * @noinspection PhpUnusedParameterInspection
@@ -124,9 +148,11 @@ class WPCaller
      * in WP function update_option().
      *
      * @deprecated
-     * @param mixed   $value      New option value.
-     * @param string  $option     Option name in db.
-     * @param mixed   $old_value  Old option value.
+     *
+     * @param mixed  $value      New option value.
+     * @param string $option     Option name in db.
+     * @param mixed  $old_value  Old option value.
+     *
      * @return mixed
      */
     public static function lock_database( $value, $option, $old_value )
@@ -148,6 +174,19 @@ class WPCaller
         }
     }
 
+    /**
+     * Redefine disabled PHP global/core functions that no longer exist
+     * (PHP 8+).
+     *
+     * @since 1.13.10
+     */
+    private static function redefineDisabledFunctions()
+    {
+        if ( version_compare(phpversion(), '8.0', '>=') ) {
+            include_once(__DIR__ . '/RedefineGlobalFuncs.php');
+        }
+    }
+
     /**
      * @return string[]
      */
@@ -160,8 +199,8 @@ class WPCaller
      * Adds key value pair to $this->outputResult to be grabbed later in
      * the $output variable of the UserCommand::issue() exec call.
      *
-     * @param string  $key
-     * @param mixed   $value
+     * @param string $key
+     * @param mixed  $value
      */
     private function outputResult( $key, $value )
     {
@@ -182,7 +221,7 @@ class WPCaller
 
         foreach ( $msgQueue as $logEntry ) {
             $label = Logger::getLvlDescr($logEntry->getLvl());
-            $debugMsgs[] = "[{$label}] {$logEntry->getMsg()}";
+            $debugMsgs[] = "[$label] {$logEntry->getMsg()}";
         }
 
         return $debugMsgs;
@@ -217,11 +256,11 @@ class WPCaller
      * WP Variables $wpdb, $table_prefix
      * WP Functions: get_option()
      *
-     * @global \wpdb    $wpdb
-     * @global string  $table_prefix
+     * @global \wpdb  $wpdb
+     * @global string $table_prefix
+     *
      * @return string
      *
-     * @noinspection PhpFullyQualifiedNameUsageInspection
      * @noinspection PhpUndefinedClassInspection
      */
     private function getSiteURL()
@@ -242,27 +281,66 @@ class WPCaller
              * Get siteurl value from DB directly.
              */
             $query = "SELECT option_value FROM {$table_prefix}options "
-                    . "WHERE option_name = 'siteurl'";
+                . "WHERE option_name = 'siteurl'";
             $siteURL = $wpdb->get_var($query);
         }
 
         return $siteURL;
     }
 
+    /**
+     *
+     * @since 1.13.4.3
+     *
+     * @return bool
+     */
+    private function generic3rdPartyAdvCachePluginExists()
+    {
+        if ( version_compare($this->installedLscwpVer, '3.0.4', '>=') ) {
+
+            /**
+             * Old LSCWP advanced-cache.php file is no longer used in these
+             * versions but is also never cleaned up. As a result, any existing
+             * advanced-cache.php files need to have their contents checked to
+             * avoid detecting an old LSCWP advanced-cache.php file as a generic
+             * 3rd-party advanced-cache plugin.
+             */
+
+            if ( file_exists($this->advancedCacheFile)
+                    && file_get_contents($this->advancedCacheFile) !== ''
+                    && !$this->advancedCacheFileHasLscacheDefine() ) {
+
+                return true;
+            }
+        }
+        elseif ( !defined('LSCACHE_ADV_CACHE') || LSCACHE_ADV_CACHE !== true ) {
+            return true;
+        }
+
+        return false;
+    }
+
     /**
      *
      * WP Functions: is_plugin_active()
      *
-     * @param WPInstall  $install
+     * @since 1.13.8  Added optional parameter $output.
+     *
+     * @param WPInstall $install
+     * @param bool      $output
+     *
      * @return int
      * @throws LSCMException  Thrown indirectly.
      */
-    private function checkStatus( WPInstall $install )
+    private function checkStatus( WPInstall $install, $output = false )
     {
-        if ( $install->getDocRoot() == null ) {
+        if ( $output == true ) {
 
             if ( ($siteUrl = $this->getSiteURL()) === '' ) {
-                Logger::uiError('Could not retrieve siteURL to match against known docroots.');
+                Logger::uiError(
+                    'Could not retrieve siteURL to match against known '
+                        . 'docroots.'
+                );
                 $install->addUserFlagFile();
 
                 return (WPInstall::ST_ERR_SITEURL | WPInstall::ST_FLAGGED);
@@ -287,16 +365,11 @@ class WPCaller
             //TODO: Get rid of ST_LSC_ADVCACHE_DEFINED status or replace with
             //      new "is caching enabled" define check.
 
-            $advCacheFile = $this->currInstall->getPath()
-                . '/wp-content/advanced-cache.php';
-
-            if ( (version_compare($this->installedLscwpVer, '3.0.4', '>=') && !file_exists($advCacheFile))
-                || (defined('LSCACHE_ADV_CACHE') && LSCACHE_ADV_CACHE === true) ) {
-
-                $status |= WPInstall::ST_LSC_ADVCACHE_DEFINED;
+            if ( $this->generic3rdPartyAdvCachePluginExists() ) {
+                $status |= WPInstall::ST_FLAGGED;
             }
             else {
-                $status |= WPInstall::ST_FLAGGED;
+                $status |= WPInstall::ST_LSC_ADVCACHE_DEFINED;
             }
         }
         else {
@@ -322,13 +395,14 @@ class WPCaller
 
     /**
      *
-     * @param boolean  $output
+     * @param boolean $output
+     *
      * @return int
      * @throws LSCMException  Thrown indirectly.
      */
     public function updateStatus( $output = false )
     {
-        $status = $this->checkStatus($this->currInstall);
+        $status = $this->checkStatus($this->currInstall, $output);
         $this->currInstall->setStatus($status);
 
         if ( $output ) {
@@ -378,7 +452,7 @@ class WPCaller
              * @noinspection PhpUndefinedConstantInspection
              * @noinspection PhpUndefinedFunctionInspection
              */
-            if ( file_exists(WP_PLUGIN_DIR . "/{$plugin}")
+            if ( file_exists(WP_PLUGIN_DIR . "/$plugin")
                     && is_plugin_active($plugin) ) {
 
                 return $slug;
@@ -393,8 +467,9 @@ class WPCaller
      * WP Constants: WP_PLUGIN_DIR
      * WP Functions: get_option()
      *
-     * @param boolean  $isMassAction
-     * @param boolean  $isNewInstall
+     * @param boolean $isMassAction
+     * @param boolean $isNewInstall
+     *
      * @return boolean
      * @throws LSCMException  Thrown indirectly.
      */
@@ -418,18 +493,20 @@ class WPCaller
             }
 
             $thirdPartyCachePluginSlug =
-                    $this->checkForKnownNonAdvCachePlugins();
+                $this->checkForKnownNonAdvCachePlugins();
 
             if ( $thirdPartyCachePluginSlug != '' ) {
                 $this->currInstall->addUserFlagFile();
 
-                $msg = 'Cannot Enable LSCWP - Detected another active cache '
-                        . "plugin \"{$thirdPartyCachePluginSlug}\". Flag set.";
-                Logger::uiError($msg);
+                Logger::uiError(
+                    'Cannot Enable LSCWP - Detected another active cache '
+                        . "plugin \"$thirdPartyCachePluginSlug\". Flag set."
+                );
 
-                $msg = 'Ignore - Detected another active cache plugin '
-                        . "\"{$thirdPartyCachePluginSlug}\". Flagged.";
-                Logger::notice($msg);
+                Logger::notice(
+                    'Ignore - Detected another active cache plugin '
+                        . "\"$thirdPartyCachePluginSlug\". Flagged."
+                );
 
                 return false;
             }
@@ -445,11 +522,11 @@ class WPCaller
             else {
                 $this->currInstall->addUserFlagFile();
 
-                $msg = 'LSCWP Already Enabled But Not Caching - Detected '
-                        . 'another active cache plugin. Please visit the '
-                        . 'WordPress Dashboard for further instructions.';
-
-                Logger::uiError($msg);
+                Logger::uiError(
+                    'LSCWP Already Enabled But Not Caching - Detected another '
+                        . 'active cache plugin. Please visit the WordPress '
+                        . 'Dashboard for further instructions.'
+                );
                 Logger::notice(
                     'Ignore - Existing install but advanced cache not set'
                 );
@@ -468,14 +545,14 @@ class WPCaller
      */
     public function directEnable()
     {
-        $isNewInstall = (file_exists($this->pluginEntry)) ? false : true;
+        $isNewInstall = !file_exists($this->pluginEntry);
 
         if ( !$this->canEnable(false, $isNewInstall) ) {
             $this->outputResult('STATUS', $this->currInstall->getStatus());
             return UserCommand::EXIT_FAIL;
         }
 
-        if (!$isNewInstall) {
+        if ( !$isNewInstall ) {
             $status = $this->enable_lscwp();
         }
         else {
@@ -500,7 +577,7 @@ class WPCaller
         /** @noinspection PhpUndefinedConstantInspection */
         $pluginDir = WP_PLUGIN_DIR;
 
-        $lscwpZip = "{$pluginDir}/litespeed-cache.latest-stable.zip";
+        $lscwpZip = "$pluginDir/litespeed-cache.latest-stable.zip";
 
         $this->downloadLSCWPZip($lscwpZip);
 
@@ -512,16 +589,18 @@ class WPCaller
         unlink($lscwpZip);
 
         if ( $unzipRet !== true ) {
-            throw new LSCMException("Unable to extract downloaded LSCWP files.",
-                    LSCMException::E_NON_FATAL);
+            throw new LSCMException(
+                "Unable to extract downloaded LSCWP files.",
+                LSCMException::E_NON_FATAL
+            );
         }
 
         $this->currInstall->addNewLscwpFlagFile();
 
         $customIni = Context::LOCAL_PLUGIN_DIR . '/'
-                . PluginVersion::LSCWP_DEFAULTS_INI_FILE_NAME;
-        $defaultIni = "{$pluginDir}/litespeed-cache/data/"
-                . PluginVersion::LSCWP_DEFAULTS_INI_FILE_NAME;
+            . PluginVersion::LSCWP_DEFAULTS_INI_FILE_NAME;
+        $defaultIni = "$pluginDir/litespeed-cache/data/"
+            . PluginVersion::LSCWP_DEFAULTS_INI_FILE_NAME;
 
         if ( file_exists($customIni) ) {
             copy($customIni, $defaultIni);
@@ -545,7 +624,8 @@ class WPCaller
      *
      * WP Constants: WP_PLUGIN_DIR
      *
-     * @param string  $lscwpZip
+     * @param string $lscwpZip
+     *
      * @return null
      * @throws LSCMException
      */
@@ -553,28 +633,31 @@ class WPCaller
     {
         /** @noinspection PhpUndefinedConstantInspection */
         $pluginDir = WP_PLUGIN_DIR;
-        $url = 'https://downloads.wordpress.org/plugin/litespeed-cache.latest-stable.zip';
+        $url = 'https://downloads.wordpress.org/plugin/'
+            . 'litespeed-cache.latest-stable.zip';
 
-        $wget_command = "wget -q --tries=1 --no-check-certificate {$url} -P {$pluginDir}";
+        $wget_command =
+            "wget -q --tries=1 --no-check-certificate $url -P $pluginDir";
         exec($wget_command, $output, $return_var);
 
         if ( $return_var === 0 && file_exists($lscwpZip) ) {
             return;
         }
 
-        $exMsg = "Failed to download LSCWP with wget exit status {$return_var}";
+        $exMsg = "Failed to download LSCWP with wget exit status $return_var";
 
         /**
-         * Fall back to curl incase wget is disabled for user.
+         * Fall back to curl in case wget is disabled for user.
          */
-        $curl_command = "cd {$pluginDir} && curl -O -s --retry 1 --insecure {$url}";
+        $curl_command =
+            "cd $pluginDir && curl -O -s --retry 1 --insecure $url";
         exec($curl_command, $output, $return_var);
 
         if ( $return_var === 0 && file_exists($lscwpZip) ) {
             return;
         }
 
-        $exMsg .= " and curl exit status {$return_var}.";
+        $exMsg .= " and curl exit status $return_var.";
 
         throw new LSCMException($exMsg, LSCMException::E_NON_FATAL);
     }
@@ -583,8 +666,9 @@ class WPCaller
      *
      * WP Constants: WP_PLUGIN_DIR
      *
-     * @param array    $extraArgs  Not used at this time.
-     * @param boolean  $massOp     True when called from massEnable().
+     * @param array   $extraArgs  Not used at this time.
+     * @param boolean $massOp     True when called from massEnable().
+     *
      * @return int
      * @throws LSCMException  Thrown indirectly.
      *
@@ -594,7 +678,7 @@ class WPCaller
     {
         /** @noinspection PhpUndefinedConstantInspection */
         $isNewInstall =
-                PluginVersion::getInstance()->prepareUserInstall(WP_PLUGIN_DIR);
+            PluginVersion::getInstance()->prepareUserInstall(WP_PLUGIN_DIR);
 
         if ( $isNewInstall ) {
             $this->installedLscwpVer = $this->getPluginVersionFromFile();
@@ -626,7 +710,8 @@ class WPCaller
 
     /**
      *
-     * @param string[]  $extraArgs
+     * @param string[] $extraArgs
+     *
      * @return int
      * @throws LSCMException  Thrown indirectly.
      */
@@ -645,7 +730,8 @@ class WPCaller
      *
      * WP Functions: is_plugin_active_for_network()
      *
-     * @param boolean  $isMassAction
+     * @param boolean $isMassAction
+     *
      * @return boolean
      * @throws LSCMException  Thrown indirectly.
      */
@@ -655,7 +741,9 @@ class WPCaller
 
         if ( $status & WPInstall::ST_PLUGIN_INACTIVE ) {
             Logger::notice('Ignore - Already disabled');
-            Logger::uiSuccess('LiteSpeed Cache Already Disabled - No Action Taken');
+            Logger::uiSuccess(
+                'LiteSpeed Cache Already Disabled - No Action Taken'
+            );
 
             return false;
         }
@@ -665,13 +753,15 @@ class WPCaller
             if ( !($status & WPInstall::ST_LSC_ADVCACHE_DEFINED) ) {
                 $this->currInstall->addUserFlagFile();
 
-                $msg = 'LSCWP Detected As Manually Enabled But Not Caching - Flag Set. If '
-                        . 'desired, this installation can be disabled from the '
-                        . 'Manage Cache Installations screen.';
-
-                Logger::uiSuccess($msg);
+                Logger::uiSuccess(
+                    'LSCWP Detected As Manually Enabled But Not Caching - Flag '
+                        . 'Set. If desired, this installation can be disabled '
+                        . 'from the Manage Cache Installations screen.'
+                );
                 Logger::notice(
-                        'Ignore for mass disable - Installed manually as advanced cache not set.');
+                    'Ignore for mass disable - Installed manually as advanced '
+                        . 'cache not set.'
+                );
 
                 return false;
             }
@@ -680,9 +770,10 @@ class WPCaller
             if ( is_plugin_active_for_network(self::LSCWP_PLUGIN) ) {
                 $this->currInstall->addUserFlagFile();
 
-                $msg = 'LiteSpeed Cache Detected As Network Activated - '
-                        . 'Flag Set & No Action Taken';
-                Logger::uiSuccess($msg);
+                Logger::uiSuccess(
+                    'LiteSpeed Cache Detected As Network Activated - Flag Set '
+                        . '& No Action Taken'
+                );
 
                 return false;
             }
@@ -693,8 +784,9 @@ class WPCaller
 
     /**
      *
-     * @param string[]  $extraArgs  Not used at this time.
-     * @param boolean   $massOp     True when called from MassDisable().
+     * @param string[] $extraArgs  Not used at this time.
+     * @param boolean  $massOp     True when called from MassDisable().
+     *
      * @return int
      * @throws LSCMException  Thrown indirectly.
      *
@@ -725,7 +817,8 @@ class WPCaller
 
     /**
      *
-     * @param string[]  $extraArgs
+     * @param string[] $extraArgs
+     *
      * @return int
      * @throws LSCMException  Thrown indirectly.
      */
@@ -753,25 +846,26 @@ class WPCaller
         $dir = WP_PLUGIN_DIR . '/litespeed-cache';
 
         if ( version_compare($this->installedLscwpVer, '3.0', '>=') ) {
-            require_once "${dir}/src/admin.cls.php";
+            require_once "$dir/src/admin.cls.php";
         }
         elseif ( version_compare($this->installedLscwpVer, '1.1.2.2', '>') ) {
-            require_once "{$dir}/admin/litespeed-cache-admin.class.php";
+            require_once "$dir/admin/litespeed-cache-admin.class.php";
         }
         else {
-            require_once "{$dir}/admin/class-litespeed-cache-admin.php";
+            require_once "$dir/admin/class-litespeed-cache-admin.php";
         }
 
         if ( version_compare($this->installedLscwpVer, '1.1.0', '<')
                 && version_compare($this->installedLscwpVer, '1.0.6', '>') ) {
 
-            require_once "{$dir}/admin/class-litespeed-cache-admin-rules.php";
+            require_once "$dir/admin/class-litespeed-cache-admin-rules.php";
         }
     }
 
     /**
      *
-     * @param boolean  $uninstall
+     * @param boolean $uninstall
+     *
      * @return int
      * @throws LSCMException  Thrown indirectly.
      */
@@ -783,9 +877,10 @@ class WPCaller
 
     /**
      *
-     * @param string[]  $fromVersions
-     * @param string    $toVersion
-     * @param boolean   $massOp        Not used at this time.
+     * @param string[] $fromVersions
+     * @param string   $toVersion
+     * @param boolean  $massOp        Not used at this time.
+     *
      * @return boolean
      *
      * @noinspection PhpUnusedParameterInspection
@@ -800,32 +895,37 @@ class WPCaller
             return false;
         }
 
-        $match = false;
+        foreach ( $fromVersions as $fromVer ) {
 
-        foreach ( $fromVersions as $ver ) {
+            $fromVerParts = explode('.', $fromVer);
+            $installedVerParts = explode('.', $this->installedLscwpVer);
 
-            if ( strpos($ver, '.x') !== false ) {
-                $ver1 = explode('.', $ver);
-                $ver2 = explode('.', $this->installedLscwpVer);
+            $i = 0;
+            $stop = sizeof($fromVerParts);
+
+            while (true) {
+
+                if ( $i == $stop || $fromVerParts[$i] == 'x' ) {
+                    return true;
+                }
+                elseif ( !isset($installedVerParts[$i])
+                        || $installedVerParts[$i] != $fromVerParts[$i] ) {
 
-                if ( $ver1[0] === $ver2[0] && $ver1[1] === $ver2[1] ) {
-                    $match = true;
                     break;
                 }
-            }
-            elseif ( $ver === $this->installedLscwpVer ) {
-                $match = true;
-                break;
+
+                $i++;
             }
         }
 
-        return $match;
+        return false;
     }
 
     /**
      *
      * @param string[] $extraArgs
-     * @param boolean $massOp
+     * @param boolean  $massOp
+     *
      * @return int
      * @throws LSCMException  Thrown indirectly.
      */
@@ -849,6 +949,7 @@ class WPCaller
     /**
      *
      * @param string[] $extraArgs
+     *
      * @return int
      * @throws LSCMException  Thrown indirectly.
      */
@@ -867,7 +968,7 @@ class WPCaller
      *
      * WP Functions: deactivate_plugins(), delete_plugins()
      *
-     * @param boolean  $uninstall
+     * @param boolean $uninstall
      */
     private function deactivate_lscwp( $uninstall )
     {
@@ -875,7 +976,7 @@ class WPCaller
         deactivate_plugins(self::LSCWP_PLUGIN);
 
         if ( $uninstall ) {
-            //add some msg about having removed plugin files?
+            //Todo: add some msg about having removed plugin files?
             /** @noinspection PhpUndefinedFunctionInspection */
             delete_plugins(array( self::LSCWP_PLUGIN ));
         }
@@ -887,12 +988,13 @@ class WPCaller
      * WP Variables: $wpdb
      * WP Functions: switch_to_blog(), restore_current_blog()
      *
-     * @global \wpdb    $wpdb
-     * @param boolean  $uninstall
+     * @global \wpdb $wpdb
+     *
+     * @param boolean $uninstall
+     *
      * @return int
      * @throws LSCMException  Thrown indirectly.
      *
-     * @noinspection PhpFullyQualifiedNameUsageInspection
      * @noinspection PhpUndefinedClassInspection
      */
     private function disable_lscwp( $uninstall )
@@ -901,7 +1003,7 @@ class WPCaller
         if ( MULTISITE ) {
             global $wpdb;
 
-            $blogs = $wpdb->get_col("SELECT blog_id FROM {$wpdb->blogs};");
+            $blogs = $wpdb->get_col("SELECT blog_id FROM $wpdb->blogs;");
 
             foreach ( $blogs as $id ) {
                 /** @noinspection PhpUndefinedFunctionInspection */
@@ -942,11 +1044,14 @@ class WPCaller
         if ( !($status & WPInstall::ST_LSC_ADVCACHE_DEFINED) ) {
             $status = $this->performDisable(true);
 
-            $msg = 'Detected another active cache plugin. Please deactivate '
-                    . 'the detected plugin and try again. You may also try '
-                    . 'manually installing through the WordPress Dashboard and '
-                    . 'following the instructions given.';
-            Logger::uiError($msg);
+            Logger::uiError(
+                'Detected '
+                    . "{$this->currInstall->getPath()}/wp-content/advanced-cache.php "
+                    . 'as belonging to another cache plugin. Please deactivate '
+                    . 'the related cache plugin and try again. You may also '
+                    . 'try manually installing through the WordPress Dashboard '
+                    . 'and following the instructions given.'
+            );
 
             $this->massIncr = 'FAIL';
         }
@@ -965,17 +1070,16 @@ class WPCaller
                 $this->currInstall->addUserFlagFile();
                 $status = $this->currInstall->getStatus();
 
-                $msg = 'LSCWP Enabled But Not Caching - Please visit the '
-                        . 'WordPress Dashboard for further instructions.';
-                Logger::uiError($msg);
-
-                $this->massIncr = 'SUCC';
+                Logger::uiError(
+                    'LSCWP Enabled But Not Caching - Please visit the '
+                        . 'WordPress Dashboard for further instructions.'
+                );
             }
             else {
                 Logger::uiSuccess('LSCWP Enabled');
-
-                $this->massIncr = 'SUCC';
             }
+
+            $this->massIncr = 'SUCC';
         }
 
         return $status;
@@ -988,8 +1092,9 @@ class WPCaller
      *               wp_clean_plugins_cache()
      * WP Classes: Plugin_Upgrader
      *
-     * @param string   $ver
-     * @param boolean  $runHooks
+     * @param string  $ver
+     * @param boolean $runHooks
+     *
      * @throws LSCMException
      */
     private function upgrade_lscwp( $ver, $runHooks = true )
@@ -1005,14 +1110,11 @@ class WPCaller
          */
         $upgrader = new \Plugin_Upgrader;
 
-        /** @noinspection PhpUndefinedMethodInspection */
         $upgrader->init();
-
-        /** @noinspection PhpUndefinedMethodInspection */
         $upgrader->upgrade_strings();
 
         $lscwpPackageURL =
-            "https://downloads.wordpress.org/plugin/litespeed-cache.{$ver}.zip";
+            "https://downloads.wordpress.org/plugin/litespeed-cache.$ver.zip";
 
         if ( $runHooks ) {
             /** @noinspection PhpUndefinedFunctionInspection */
@@ -1033,20 +1135,21 @@ class WPCaller
         }
 
         /**
-         * @noinspection PhpUndefinedMethodInspection
          * @noinspection PhpUndefinedConstantInspection
          */
-        $upgrader->run(array(
-            'package' => $lscwpPackageURL,
-            'destination' => WP_PLUGIN_DIR,
-            'clear_destination' => true,
-            'clear_working' => true,
-            'hook_extra' => array(
-                'plugin' => $this->pluginEntry,
-                'type' => 'plugin',
-                'action' => 'update',
+        $upgrader->run(
+            array(
+                'package' => $lscwpPackageURL,
+                'destination' => WP_PLUGIN_DIR,
+                'clear_destination' => true,
+                'clear_working' => true,
+                'hook_extra' => array(
+                    'plugin' => $this->pluginEntry,
+                    'type' => 'plugin',
+                    'action' => 'update',
+                )
             )
-        ));
+        );
 
         /**
          * Start new messages on a new line
@@ -1073,12 +1176,13 @@ class WPCaller
         }
 
         /**
-         * @noinspection PhpUndefinedFieldInspection
          * @noinspection PhpUndefinedFunctionInspection
          */
         if ( !$upgrader->result || is_wp_error($upgrader->result) ) {
-            throw new LSCMException("Failed to upgrade to v{$ver}.",
-            LSCMException::E_NON_FATAL);
+            throw new LSCMException(
+                "Failed to upgrade to v$ver.",
+                LSCMException::E_NON_FATAL
+            );
         }
 
         $this->updateTranslationFiles();
@@ -1109,7 +1213,7 @@ class WPCaller
     /**
      * Checks for local plugin translation files and copies them to the plugin
      * languages directory if able. This function will also attempt to inform
-     * the root user when a locales translation should be retrieved or removed.
+     * the root user when a locale's translation should be retrieved or removed.
      *
      * WP Functions: get_locale(), unzip_file()
      * WP Classes: WP_Filesystem
@@ -1124,15 +1228,15 @@ class WPCaller
         }
 
         $localTranslationDir = Context::LOCAL_PLUGIN_DIR
-                . "/{$this->installedLscwpVer}/translations";
+            . "/$this->installedLscwpVer/translations";
 
-        $moFileName = "litespeed-cache-{$locale}.mo";
-        $poFileName = "litespeed-cache-{$locale}.po";
-        $localMoFile = "{$localTranslationDir}/{$moFileName}";
-        $localPoFile = "{$localTranslationDir}/{$poFileName}";
-        $zipFile = "{$localTranslationDir}/{$locale}.zip";
-        $translationFlag = "{$localTranslationDir}/"
-                . PluginVersion::TRANSLATION_CHECK_FLAG_BASE . "_{$locale}";
+        $moFileName = "litespeed-cache-$locale.mo";
+        $poFileName = "litespeed-cache-$locale.po";
+        $localMoFile = "$localTranslationDir/$moFileName";
+        $localPoFile = "$localTranslationDir/$poFileName";
+        $zipFile = "$localTranslationDir/$locale.zip";
+        $translationFlag = "$localTranslationDir/"
+            . PluginVersion::TRANSLATION_CHECK_FLAG_BASE . "_$locale";
 
         $langDir =
             $this->currInstall->getPath() . '/wp-content/languages/plugins';
@@ -1142,8 +1246,8 @@ class WPCaller
         }
 
         if ( file_exists($localMoFile) && file_exists($localPoFile) ) {
-            copy($localMoFile, "{$langDir}/{$moFileName}");
-            copy($localPoFile, "{$langDir}/{$poFileName}");
+            copy($localMoFile, "$langDir/$moFileName");
+            copy($localPoFile, "$langDir/$poFileName");
         }
         elseif ( file_exists($zipFile) ) {
             /**
@@ -1154,8 +1258,10 @@ class WPCaller
 
             /** @noinspection PhpUndefinedFunctionInspection */
             if ( unzip_file($zipFile, $langDir) !== true ) {
-                $this->outputResult('BAD_TRANSLATION',
-                        "{$locale} {$this->installedLscwpVer}");
+                $this->outputResult(
+                    'BAD_TRANSLATION',
+                    "$locale $this->installedLscwpVer"
+                );
             }
         }
         elseif ( !file_exists($translationFlag) ||
@@ -1163,34 +1269,53 @@ class WPCaller
 
             $this->outputResult(
                 'GET_TRANSLATION',
-                "{$locale} {$this->installedLscwpVer}"
+                "$locale $this->installedLscwpVer"
             );
         }
     }
 
+    /**
+     *
+     * @since 1.13.4.4
+     *
+     * @return bool
+     */
+    private function advancedCacheFileHasLscacheDefine()
+    {
+        $content = file_get_contents($this->advancedCacheFile);
+
+        if ( strpos($content, 'LSCACHE_ADV_CACHE') !== false ) {
+            return true;
+        }
+
+        return false;
+    }
+
     private function includeLSCWPAdvancedCacheFile()
     {
-        $advCacheFile = $this->currInstall->getPath()
-                . '/wp-content/advanced-cache.php';
+        if ( file_exists($this->advancedCacheFile) ) {
 
-        if ( file_exists($advCacheFile) ) {
-            $content = file_get_contents($advCacheFile);
-
-            if ( strpos($content, 'LSCACHE_ADV_CACHE') !== false ) {
+            if ( $this->advancedCacheFileHasLscacheDefine() ) {
                 /** @noinspection PhpIncludeInspection */
-                include_once $advCacheFile;
+                include_once $this->advancedCacheFile;
             }
         }
     }
 
+    /**
+     *
+     * @throws LSCMException  Thrown indirectly.
+     */
     public function removeLscwpPluginFiles()
     {
         $this->currInstall->removePluginFiles(dirname($this->pluginEntry));
     }
 
     /**
-     * @param string[]  $extraArgs
-     * @param boolean   $massOp     Not used at this time.
+     *
+     * @param string[] $extraArgs
+     * @param boolean  $massOp     Not used at this time.
+     *
      * @return int
      * @throws LSCMException  Thrown indirectly.
      *
@@ -1206,7 +1331,7 @@ class WPCaller
             DashNotifier::prepareUserInstall();
 
             if ( DashNotifier::doNotify($jsonInfo) ) {
-                Logger::uiSuccess('Notfied Successfully');
+                Logger::uiSuccess('Notified Successfully');
                 $this->massIncr = 'SUCC';
                 $ret = UserCommand::EXIT_SUCC;
             }
@@ -1224,7 +1349,8 @@ class WPCaller
 
     /**
      *
-     * @param string[]  $extraArgs
+     * @param string[] $extraArgs
+     *
      * @return int
      * @throws LSCMException  Thrown indirectly.
      */
@@ -1246,13 +1372,14 @@ class WPCaller
      * WP Functions: switch_to_blog(), restore_current_blog(),
      *               is_plugin_active()
      *
-     * @global \wpdb    $wpdb
-     * @param string[]  $extraArgs  Unused for now.
-     * @param boolean   $massOp
+     * @global \wpdb $wpdb
+     *
+     * @param string[] $extraArgs  Unused for now.
+     * @param boolean  $massOp
+     *
      * @return int
      *
      * @noinspection PhpUndefinedClassInspection
-     * @noinspection PhpFullyQualifiedNameUsageInspection
      * @noinspection PhpUnusedParameterInspection
      */
     public function dashDisable( $extraArgs, $massOp = false )
@@ -1261,7 +1388,7 @@ class WPCaller
         if ( MULTISITE ) {
             global $wpdb;
 
-            $blogs = $wpdb->get_col("SELECT blog_id FROM {$wpdb->blogs};");
+            $blogs = $wpdb->get_col("SELECT blog_id FROM $wpdb->blogs;");
 
             foreach ( $blogs as $id ) {
                 /** @noinspection PhpUndefinedFunctionInspection */
@@ -1293,7 +1420,8 @@ class WPCaller
 
     /**
      *
-     * @param array  $extraArgs
+     * @param array $extraArgs
+     *
      * @return int
      */
     public function massDashDisable( $extraArgs )
@@ -1313,7 +1441,8 @@ class WPCaller
      *
      * @since 1.12
      *
-     * @param boolean  $setOutputResult
+     * @param boolean $setOutputResult
+     *
      * @return string
      */
     public function getQuicCloudAPIKey( $setOutputResult = false )
@@ -1337,13 +1466,13 @@ class WPCaller
      *
      * @since 1.9.8
      *
-     * @param string  $key
-     * @param mixed   $val
+     * @param string $key
+     * @param mixed  $val
      */
     private function setEnvVar( $key, $val )
     {
         $_SERVER[$key] = $val;
-        putenv("{$key}={$val}");
+        putenv("$key=$val");
     }
 
     /**
@@ -1369,7 +1498,7 @@ class WPCaller
         $pattern5 = '/define\(\s*[\'"]SUNRISE[\'"]\s*,[^;]*;/';
 
         $config_content =
-                file_get_contents($this->currInstall->getWpConfigFile());
+            file_get_contents($this->currInstall->getWpConfigFile());
 
         if ( preg_match($pattern1, $config_content, $m1)
                 && preg_match($pattern2, $m1[0]) ) {
@@ -1385,20 +1514,24 @@ class WPCaller
 
         if ( $isMultiSite ) {
 
-            if ( !preg_match('/define\(\s*[\'"]DOMAIN_CURRENT_SITE[\'"]\s*,\s*[\'"](.+)[\'"]\s*\)\s*;/',
-                    $config_content, $m2) ) {
+            $domainPattern = '/define\(\s*[\'"]DOMAIN_CURRENT_SITE[\'"]\s*,'
+                . '\s*[\'"](.+)[\'"]\s*\)\s*;/';
 
+            if ( !preg_match($domainPattern, $config_content, $m2) ) {
                 throw new LSCMException(
-                        'Cannot find DOMAIN_CURRENT_SITE with MULTISITE defined.');
+                    'Cannot find DOMAIN_CURRENT_SITE with MULTISITE defined.'
+                );
             }
 
             $this->currInstall->setServerName($m2[1]);
 
-            if ( !preg_match('/define\(\s*[\'"]PATH_CURRENT_SITE[\'"]\s*,\s*[\'"](.+)[\'"]\s*\)\s*;/',
-                    $config_content, $m3) ) {
+            $pathPattern = '/define\(\s*[\'"]PATH_CURRENT_SITE[\'"]\s*,'
+                . '\s*[\'"](.+)[\'"]\s*\)\s*;/';
 
+            if ( !preg_match($pathPattern, $config_content, $m3) ) {
                 throw new LSCMException(
-                        'Cannot find PATH_CURRENT_SITE with MULTISITE defined.');
+                    'Cannot find PATH_CURRENT_SITE with MULTISITE defined.'
+                );
             }
 
             $this->setEnvVar('REQUEST_URI', $m3[1]);
@@ -1416,8 +1549,9 @@ class WPCaller
      * WP Functions: wp_plugin_directory_constants(), wp_cookie_constants()
      * WP Classes: WP_Query
      *
-     * @global \wpdb  $wpdb
-     * @global array  $shortcode_tags
+     * @global \wpdb $wpdb
+     * @global array $shortcode_tags
+     *
      * @throws LSCMException  Thrown directly and indirectly.
      *
      * @noinspection PhpUndefinedClassInspection
@@ -1452,12 +1586,12 @@ class WPCaller
          *
          * @noinspection PhpIncludeInspection
          */
-        include_once "{$wpPath}/wp-includes/version.php";
+        include_once "$wpPath/wp-includes/version.php";
 
         /** @noinspection PhpUndefinedVariableInspection */
         if ( version_compare($wp_version, '4.0', '<') ) {
             throw new LSCMException(
-                "Detected WordPress version as {$wp_version}. Version 4.0 '
+                "Detected WordPress version as $wp_version. Version 4.0 '
                     . 'required at minimum."
             );
         }
@@ -1465,7 +1599,7 @@ class WPCaller
         /**
          * Set needed server variables.
          */
-        $_SERVER['SCRIPT_FILENAME'] = "{$wpPath}/wp-admin/plugins.php";
+        $_SERVER['SCRIPT_FILENAME'] = "$wpPath/wp-admin/plugins.php";
 
         if ( ! $this->isMultisite() ) {
             $this->setEnvVar('REQUEST_URI', '');
@@ -1574,11 +1708,15 @@ class WPCaller
          */
         unset($_SERVER['REQUEST_METHOD']);
 
+        self::redefineDisabledFunctions();
+
         foreach ( $includeFiles as $file ) {
             $file = $wpPath . $file;
 
             if ( !file_exists($file) ) {
-                throw new LSCMException("Could not include missing file {$file}.");
+                throw new LSCMException(
+                    "Could not include missing file $file."
+                );
             }
 
             /** @noinspection PhpIncludeInspection */
