@@ -118,7 +118,8 @@ class CValidation
 			}
 		}
 
-		$checkedTids = array( 'VH_TOP_D', 'VH_BASE', 'VH_UDB', 'ADMIN_USR', 'ADMIN_USR_NEW',
+		$checkedTids = array( 'VH_TOP_D', 'VH_BASE', 'VH_UDB', 
+            'ADMIN_USR', 'ADMIN_USR_NEW',
             'L_GENERAL', 'L_GENERAL1', 'ADMIN_L_GENERAL', 'ADMIN_L_GENERAL1',   // WS
             'L_GENERAL_NEW', 'L_GENERAL_DEL', 'LT_GENERAL_NEW', 'LT_GENERAL', // LSLB
             'L_SSL_CERT',
@@ -200,7 +201,7 @@ class CValidation
 		return $isValid;
 	}
 
-	protected function chkPostTbl_VH_UDB(&$d)
+    protected function chkPostTbl_VH_UDB(&$d)
 	{
 		$isValid = 1;
 		if ( $d['pass']->GetVal() != $d['pass1']->GetVal() ) {
@@ -437,7 +438,7 @@ class CValidation
 		}
 
 		$chktype = array('uint', 'name', 'vhname', 'dbname', 'sel','sel1','sel2',
-		'bool','file','filep','file0','file1', 'filetp', 'path',
+		'bool','file','filep','file0','file1', 'filetp', 'path', 'note',
 		'uri','expuri','url', 'httpurl', 'email', 'dir', 'addr', 'ipport', 'wsaddr', 'parse');
 
 		if ( !in_array($attr->_type, $chktype) )	{
@@ -501,7 +502,7 @@ class CValidation
 
 	protected function chkAttr_name_val($attr, $val, &$err)
 	{
-		if ( preg_match( "/[<>&%]/", $val) ) {
+		if ( preg_match( "/[{}<>&%]/", $val) ) {
 			$err = 'invalid characters in name';
 			return -1;
 		}
@@ -516,7 +517,7 @@ class CValidation
 	{
 		$cval->SetVal(preg_replace("/\s+/", ' ', $cval->GetVal()));
 		$val = $cval->GetVal();
-		if ( preg_match( "/[,;<>&%]/", $val ) ) {
+		if ( preg_match( "/[,;<>&%=\(\)\"']/", $val ) ) {
 			$cval->SetErr('Invalid characters found in name');
 			return -1;
 		}
@@ -531,6 +532,15 @@ class CValidation
 		return 1;
 	}
 
+    protected function chkAttr_note($attr, $cval)
+    {
+        if (preg_match("/[{}<]/", $cval->GetVal(), $m)) { // avoid <script, also {} for conf format
+            $cval->SetErr("character $m[0] not allowed");
+            return -1;
+        }
+        return 1;
+    }
+
 	protected function chkAttr_vhname($attr, $cval)
 	{
         $res = $this->chkAttr_dbname($attr, $cval);
@@ -571,16 +581,23 @@ class CValidation
 		return FALSE;
 	}
 
+    protected function clean_absolute_path($abspath)
+    {
+        $absname = PathTool::clean($abspath);
+        if ( isset( $_SERVER['LS_CHROOT'] ) )	{
+            $root = $_SERVER['LS_CHROOT'];
+            $len = strlen($root);
+            if ( strncmp( $absname, $root, $len ) == 0 ) {
+                $absname = substr($absname, $len);
+            }
+        }
+        return $absname;
+    }
+
+
 	protected function test_file(&$absname, &$err, $attr)
 	{
-		$absname = PathTool::clean($absname);
-		if ( isset( $_SERVER['LS_CHROOT'] ) )	{
-			$root = $_SERVER['LS_CHROOT'];
-			$len = strlen($root);
-			if ( strncmp( $absname, $root, $len ) == 0 ) {
-				$absname = substr($absname, $len);
-			}
-		}
+        $absname = $this->clean_absolute_path($absname);
 
 		if ( $attr->_type == 'file0' ) {
 			if ( !file_exists($absname) ) {
@@ -664,6 +681,23 @@ class CValidation
         }
         return false;
     }
+
+	protected function check_cmd_invalid_str($cmd)
+	{
+		// check if it's allowed command, do not allow ' " -c -i /dev/tcp bash sh csh tcsh zsh ksh
+		$cmd = str_replace('.', 'a', $cmd); // replace . with char before pattern check
+		$pattern = '#("|\'|;|-c|-i|/dev/tcp|curl|wget|fetch|\Wbash\W|\Wsh\W|\Wcsh\W|\Wtcsh\W|\Wzsh\W|\Wksh\W)#';
+
+
+		if (preg_match($pattern, $cmd, $m)) {
+			return $m[0];
+		}
+		$cmd = str_replace('\\', '', $cmd); // remove all escape & try again
+		if (preg_match($pattern, $cmd, $m)) {
+			return $m[0];
+		}
+		return null;
+	}
     
 	public function chkAttr_file_val($attr, $val, &$err)
 	{
@@ -684,12 +718,17 @@ class CValidation
 			$path = substr($val, 0, strrpos($val,'/'));
 		} else {
 			$path = $val;
-			if ( $attr->_type == 'file1' ) {
-				$pos = strpos($val, ' ');
-				if ( $pos > 0 ) {
-					$path = substr($val, 0, $pos);
+            if ($attr->_type == 'file1') { // file1 is command
+				$invalid_str = $this->check_cmd_invalid_str($path);
+				if ($invalid_str) {
+					$err = 'Cannot contain string ' . htmlspecialchars($invalid_str, ENT_QUOTES);
+					return -1;
 				}
-			}
+                $pos = strpos($val, ' ');
+                if ($pos > 0) {
+                    $path = substr($val, 0, $pos); // check first part is valid path
+                }
+            }
 		}
 
 		$res = $this->chk_file1($attr, $path, $err);
@@ -733,27 +772,43 @@ class CValidation
 	}
 
 	protected function chk_file1($attr, &$path, &$err)
+	{
+        if ($this->get_abs_path($attr->_minVal, $path, $err) == 1) {
+			return $this->test_file($path, $err, $attr);
+		}
+	}
+
+    protected function get_cleaned_abs_path($attr_minVal, &$path, &$err)
+    {
+        if ($this->get_abs_path($attr_minVal, $path, $err) == 1) {
+            $absname = $this->clean_absolute_path($path);
+            return $absname;
+        }
+        return null;
+    }
+
+	protected function get_abs_path($attr_minVal, &$path, &$err)
 	{
 		if(!strlen($path)) {
 			$err = "Invalid Path.";
 			return -1;
 		}
 
-		$s = $path{0};
+		$s = substr($path, 0, 1);
 
 		if ( strpos($path, '$VH_NAME') !== FALSE )	{
 			$path = str_replace('$VH_NAME', $this->_info['VH_NAME'], $path);
 		}
 
 		if ( $s == '/' ) {
-			return $this->test_file($path, $err, $attr);
+			return 1;
 		}
 
-		if ( $attr->_minVal == 1 ) {
+		if ( $attr_minVal == 1 ) {
 			$err = 'only accept absolute path';
 			return -1;
 		}
-		elseif ( $attr->_minVal == 2 ) {
+		elseif ( $attr_minVal == 2 ) {
             if (strncasecmp('$SERVER_ROOT', $path, 12) == 0) {
                 $path = $_SERVER['LS_SERVER_ROOT'] . substr($path, 13);
             } elseif ($s == '$') {
@@ -763,7 +818,7 @@ class CValidation
                 $path = $_SERVER['LS_SERVER_ROOT'] . $path; // treat as relative to SERVER_ROOT
             }
 		}
-		elseif ( $attr->_minVal == 3 ) {
+		elseif ( $attr_minVal == 3 ) {
 			if ( strncasecmp('$SERVER_ROOT', $path, 12) == 0 ) {
 				$path = $_SERVER['LS_SERVER_ROOT'] . substr($path, 13);
 			} elseif ( strncasecmp('$VH_ROOT', $path, 8) == 0 )	{
@@ -777,7 +832,7 @@ class CValidation
                 $path = $_SERVER['LS_SERVER_ROOT'] . $path; // treat as relative to SERVER_ROOT
             }
 		}
-		elseif ( $attr->_minVal == 4 ) {
+		elseif ( $attr_minVal == 4 ) {
 			if ( strncasecmp('$SERVER_ROOT', $path, 12) == 0 ) {
 				$path = $_SERVER['LS_SERVER_ROOT'] . substr($path, 13);
 			} elseif ( strncasecmp('$VH_ROOT', $path, 8) == 0 )	{
@@ -789,9 +844,10 @@ class CValidation
 			}
 		}
 
-		return $this->test_file($path, $err, $attr);
+		return 1;
 	}
 
+
 	protected function chkAttr_uri($attr, $cval)
 	{
 		$val = $cval->GetVal();
@@ -805,7 +861,7 @@ class CValidation
 	protected function chkAttr_expuri($attr, $cval)
 	{
 		$val = $cval->GetVal();
-		if ( $val{0} == '/' || strncmp( $val, 'exp:', 4 ) == 0 ) {
+		if ( substr($val, 0, 1) == '/' || strncmp( $val, 'exp:', 4 ) == 0 ) {
 			return 1;
 		} else {
 			$cval->SetErr('URI must start with "/" or "exp:"');
@@ -816,7 +872,7 @@ class CValidation
 	protected function chkAttr_url($attr, $cval)
 	{
 		$val = $cval->GetVal();
-		if (( $val{0} != '/' )
+		if (( substr($val, 0, 1) != '/' )
 		&&( strncmp( $val, 'http://', 7 ) != 0 )
 		&&( strncmp( $val, 'https://', 8 ) != 0 )) {
 			$cval->SetErr('URL must start with "/" or "http(s)://"');
@@ -853,21 +909,30 @@ class CValidation
 		}
 	}
 
+	protected function isUdsAddr($v)
+	{
+		// check UDS:// unix:
+		if (preg_match('/^(UDS:\/\/|unix:)(.+)$/i', $v, $m)) {
+            $v = $m[2];
+            $supportedvar = ['$SERVER_ROOT', '$VH_NAME', '$VH_ROOT', '$DOC_ROOT'];
+            $v = str_replace($supportedvar, 'VAR', $v);
+            if (preg_match("/^[a-z0-9\-_\/\.]+$/i", $v)) {
+                return 1;
+            }
+		}
+		return 0;
+	}
+
 	protected function chkAttr_addr($attr, $cval)
 	{
         $v = $cval->GetVal();
         if (preg_match("/^([[:alnum:]._-]+|\[[[:xdigit:]:]+\]):(\d)+$/", $v)) {
             return 1;
-        } 
-        if (stripos($v, 'UDS://') !== false) {
-            $v = substr($v, 6);
-            $supportedvar = ['$SERVER_ROOT', '$VH_NAME', '$VH_ROOT', '$DOC_ROOT'];
-            $v = str_replace($supportedvar, 'VAR', $v);
-            if (preg_match("/^[a-z0-9\-_\/\.]+$/i", $v)) {
-                return 1;
-            } 
         }
-        $cval->SetErr('invalid address: correct syntax is "IPV4|IPV6_address:port" or UDS://path');
+		if ($this->isUdsAddr($v)) {
+            return 1;
+        }
+        $cval->SetErr('invalid address: correct syntax is "IPV4|IPV6_address:port" or UDS://path or unix:path');
         return -1;
 	}
 
@@ -883,12 +948,16 @@ class CValidation
 
     protected function chkAttr_wsaddr($attr, $cval)
 	{
-		if ( preg_match("/^((http|https):\/\/)?([[:alnum:]._-]+|\[[[:xdigit:]:]+\])(:\d+)?$/", $cval->GetVal()) ) {
+		$v = $cval->GetVal();
+		if ( preg_match("/^((http|https):\/\/)?([[:alnum:]._-]+|\[[[:xdigit:]:]+\])(:\d+)?$/", $v) ) {
 			return 1;
-		} else {
-			$cval->SetErr('invalid address: correct syntax is "[http|https://]IPV4|IPV6_address[:port]". ');
-			return -1;
 		}
+		if ($this->isUdsAddr($v)) {
+            return 1;
+        }
+
+		$cval->SetErr('invalid address: correct syntax is "[http|https://]IPV4|IPV6_address[:port]" or Unix Domain Socket address "UDS://path or unix:path".');
+		return -1;
 	}
 
 	protected function chkAttr_bool($attr, $cval)
@@ -914,7 +983,11 @@ class CValidation
 		if ( preg_match($attr->_minVal, $val) ) {
 			return 1;
 		} else {
-			$err = "invalid format - $val, please check syntax";
+			if ($attr->_maxVal) {
+				$err = "invalid format - $val, please check syntax";
+			} else {
+				$err = "invalid value - $val";
+			}
 			return -1;
 		}
 	}
