@@ -214,7 +214,7 @@ class ConfCenter
         if ( $vn != NULL && strpos($filename, '$VH_NAME') !== false ) {
             $filename = str_replace('$VH_NAME', $vn, $filename) ;
         }
-        $s = $filename{0} ;
+        $s = substr($filename, 0, 1) ;
         if ( $s == '$' ) {
             if ( strncasecmp('$SERVER_ROOT', $filename, 12) == 0 ) {
                 $filename = $_SERVER['LS_SERVER_ROOT'] . substr($filename, 13) ;
@@ -805,7 +805,7 @@ class ConfCenter
 
         $d = $this->_curOne->_data ;
         $configFile = $d['general']['configFile'] ;
-        if ( $configFile->HasErr() ) {
+		if ( empty($configFile) || $configFile->HasErr() ) {
             $disp->_err = 'Please fix error in General->Config File first.' ;
             return false ;
         }
