@@ -55,7 +55,7 @@ class Service
 			if ( !isset($this->vhosts[$vhname]) )
 				$this->vhosts[$vhname] = '0A';
             else
-				$this->vhosts[$vhname] = $this->vhosts[$vhname]{0} . 'M';
+				$this->vhosts[$vhname] = substr($this->vhosts[$vhname], 0, 1) . 'M';
 		}
 
 		$this->serv['name'] = $data['serv'];
@@ -164,7 +164,7 @@ class Service
 				$this->readVh($buffer, $fd);
 			}
             elseif (strncmp($buffer, 'DEBUG_LOG: ', 11) == 0) {
-                if ($buffer{11} === '1')
+                if (substr($buffer, 11, 1) === '1')
                     $this->debugOn = true;
                 else
                     $this->debugOn = false;
@@ -566,7 +566,7 @@ class Service
 	{
 		$newlineTag = '[ERR[WAR[NOT[INF[DEB';
 		$levels = array( 'E' => 1, 'W' => 2, 'N' => 3, 'I' => 4, 'D' => 5 );
-		$level = $levels[$data['level']{0}];
+		$level = $levels[substr($data['level'], 0, 1)];
 
 		$fd = fopen($data['filename'], 'r');
 		if ( !$fd )
@@ -607,7 +607,7 @@ class Service
 					$result .= '</td></tr>' . "\n";
 					$start = 0;
 				}
-				$b28 = $c28{0};
+				$b28 = substr($c28, 0, 1);
 				if ( $levels[$b28] <= $level )
 				{
 					// start a new line
@@ -711,22 +711,11 @@ class Service
 
 	public static function GetLoadAvg()
 	{
-		$load_avg = 'N/A';
-		$UPTIME = '';
-		$UPTIME = exec("uptime");
-		$UPTIME = strtolower( $UPTIME );
-		$pos = strpos($UPTIME, "load average");
-		if ( $pos > 0 )
-		{
-			$pos += 13;
-            if ( $UPTIME[$pos] == 's' )
-	            ++$pos;
-
-			if ( $UPTIME[$pos] == ':' )
-				++$pos;
-			$load_avg = trim(substr($UPTIME, $pos ));
+		$avgload = \sys_getloadavg();
+		if ($avgload === false) {
+			return 'N/A';
 		}
-		return $load_avg;
+		return implode(', ', array_map(function($load) { return round($load, 3); }, $avgload));
 	}
 
 }
