@@ -38,6 +38,11 @@ if [ $? -ne 0 ]; then
     find "/tmp/lshttpd/swap" -type s -atime +1 2>/dev/null | xargs rm -f
 fi 
 
+find "/dev/shm/lsws/ocspcache" -type s -mtime +7 -delete 2>/dev/null
+if [ $? -ne 0 ]; then
+    find "/dev/shm/lsws/ocspcache" -type s -mtime +7 2>/dev/null | xargs rm -f
+fi
+
 #if [ -f '/usr/bin/ionice' ]; then
 #    echo "ionice:" `ionice` 1>&2
 #fi 
