@@ -1,4 +1,4 @@
-#!/bin/sh
+#!/bin/bash
 
 ##############################################################################
 # Switch between Apache and LiteSpeed Web Server under control panel environment
@@ -54,11 +54,7 @@ detect_control_panel()
         CP="DA"
         DA_DIR="/usr/local/directadmin/custombuild"
 
-        if [[ `/usr/local/directadmin/custombuild/build version` =~ ^2* ]] ; then
-            echo "Detected DirectAdmin Environment"
-        else
-            check_errs 1 "DirectAdmin environment detected, but CustomBuild 2.0 not available"
-        fi
+        echo "Detected DirectAdmin Environment"
     # detect CentOS Web Panel
     elif [ -d "/usr/local/cwpsrv" ] ; then
         CP="CWP"
@@ -200,6 +196,13 @@ cp_start_fake_apache()
     done
 }
 
+remove_bak_bak()
+{
+    if [ -f "$1$2" ]; then
+        mv "$1$2" "$1$2_restore_done"
+    fi
+}
+
 cpanel_apache_wrapper_helper()
 {
     RESTORE=$1
@@ -213,18 +216,21 @@ cpanel_apache_wrapper_helper()
         elif [ -f "/usr/local/apache/bin/${AP_PROC}_ls_bak_bak" ] ; then
             cp -f "/usr/local/apache/bin/${AP_PROC}_ls_bak_bak" "/usr/local/apache/bin/${AP_PROC}"
         fi
+        remove_bak_bak "/usr/local/apache/bin/${AP_PROC}" "_ls_bak_bak"
 
         if [ -f "/usr/sbin/${AP_PROC}_ls_bak" ] ; then
             mv -f "/usr/sbin/${AP_PROC}_ls_bak" "/usr/sbin/${AP_PROC}"
         elif [ -f "/usr/sbin/${AP_PROC}_ls_bak_bak" ] ; then
             cp -f "/usr/sbin/${AP_PROC}_ls_bak_bak" "/usr/sbin/${AP_PROC}"
         fi
+        remove_bak_bak "/usr/sbin/${AP_PROC}" "_ls_bak_bak"
 
         if [ -f "/scripts/restartsrv_httpd_ls_bak" ] ; then
             mv -f "/scripts/restartsrv_httpd_ls_bak" "/scripts/restartsrv_httpd"
         elif [ -f "/scripts/restartsrv_httpd_ls_bak_bak" ] ; then
             cp -f "/scripts/restartsrv_httpd_ls_bak_bak" "/scripts/restartsrv_httpd"
         fi
+        remove_bak_bak "/scripts/restartsrv_httpd" "_ls_bak_bak"
 
         if [ $NEED_PROTECT -eq 1 ] ; then
             /scripts/enablefileprotect
@@ -243,6 +249,7 @@ cpanel_apache_wrapper_helper()
             elif [  -e "${SYSTEMDDIR}/${AP_PROC}.service.ls_bak_bak" ]  ; then
                 cp -f "${SYSTEMDDIR}/${AP_PROC}.service.ls_bak_bak" "${SYSTEMDDIR}/${AP_PROC}.service"
             fi
+            remove_bak_bak "${SYSTEMDDIR}/${AP_PROC}.service" ".ls_bak_bak"
             systemctl daemon-reload
         fi
 
@@ -252,6 +259,7 @@ cpanel_apache_wrapper_helper()
             elif [ -e "${INIT_DIR}/${AP_PROC}.ls_bak_bak" ] ; then
                 cp -f "${INIT_DIR}/${AP_PROC}.ls_bak_bak" "${INIT_DIR}/${AP_PROC}"
             fi
+            remove_bak_bak "${INIT_DIR}/${AP_PROC}" ".ls_bak_bak"
         fi
 
         if [ -f "$LSWS_SWITCH_FLAG" ] ; then
@@ -308,6 +316,7 @@ plesk_apache_wrapper_helper()
         elif [ -f "/usr/sbin/${AP_PROC}_ls_bak_bak" ] ; then
             cp -f "/usr/sbin/${AP_PROC}_ls_bak_bak" "/usr/sbin/${AP_PROC}"
         fi
+        remove_bak_bak "/usr/sbin/${AP_PROC}" "_ls_bak_bak"
 
         # restore rc file
         if [ "$SYSTEMDDIR" != "" ]; then
@@ -316,6 +325,12 @@ plesk_apache_wrapper_helper()
             elif [ -e "${SYSTEMDDIR}/${AP_PROC}.service.ls_bak_bak" ]  ; then
                 cp -f "${SYSTEMDDIR}/${AP_PROC}.service.ls_bak_bak" "${SYSTEMDDIR}/${AP_PROC}.service"
             fi
+            remove_bak_bak "${SYSTEMDDIR}/${AP_PROC}.service" ".ls_bak_bak"
+
+            if [ -d "${SYSTEMDDIR}/lshttpd.service.d" ]; then
+                mv -f "${SYSTEMDDIR}/lshttpd.service.d" "${SYSTEMDDIR}/${AP_PROC}.service.d"
+            fi
+
             systemctl daemon-reload
             systemctl disable lshttpd
             systemctl enable ${AP_PROC}
@@ -327,7 +342,8 @@ plesk_apache_wrapper_helper()
             elif [ -e "${INIT_DIR}/${AP_PROC}.ls_bak_bak" ]  ; then
                 cp -f "${INIT_DIR}/${AP_PROC}.ls_bak_bak" "${INIT_DIR}/${AP_PROC}"
             fi
-            if [[ ${SYSTEMDEXIST} -eq 1 ]] ; then
+            remove_bak_bak "${INIT_DIR}/${AP_PROC}.service" ".ls_bak_bak"
+            if [ ${SYSTEMDEXIST} -eq 1 ] ; then
                 systemctl daemon-reload
             fi
         fi
@@ -346,6 +362,11 @@ plesk_apache_wrapper_helper()
             fi
             
             ln -sf "${SYSTEMDDIR}/lshttpd.service" "${SYSTEMDDIR}/${AP_PROC}.service"
+
+            if [ -d "${SYSTEMDDIR}/${AP_PROC}.service.d" ]; then
+                mv -f "${SYSTEMDDIR}/${AP_PROC}.service.d" "${SYSTEMDDIR}/lshttpd.service.d"
+            fi
+
             systemctl daemon-reload
         fi
 
@@ -358,7 +379,7 @@ plesk_apache_wrapper_helper()
             cp "${INIT_DIR}/lsws" "${INIT_DIR}/${AP_PROC}"
             #ln -sf ./lsws "${INIT_DIR}/${AP_PROC}"
             
-            if [[ ${SYSTEMDEXIST} -eq 1 ]] ; then
+            if [ ${SYSTEMDEXIST} -eq 1 ] ; then
                 systemctl daemon-reload
             fi
         fi
@@ -610,7 +631,7 @@ switch_da()
         pkill wswatch.sh
         ./build set webserver apache
         OS_NAME=$(cat /etc/redhat-release | awk '{print $1}')
-        if [[ ${OS_NAME} == 'CloudLinux' ]] ; then
+        if [ ${OS_NAME} == 'CloudLinux' ] ; then
             ./build set php1_mode lsphp
             ./build set php2_mode lsphp
         else
