@@ -657,7 +657,6 @@ buildAdminSslCert()
 {
     if [ ! -f "$LSWS_HOME/admin/conf/cert/admin.crt" ]; then
         HN=`hostname`
-        detect_dlcmd
         if [ "x${DLCMD}" != 'xx' ]; then
             $DLCMD $LSWS_HOME/myip http://cyberpanel.sh/?ip
             MYIP=`cat $LSWS_HOME/myip`
@@ -669,6 +668,14 @@ buildAdminSslCert()
 }
 
 
+download_quic_cloud_ips()
+{
+    if [ "x${DLCMD}" != 'xx' ]; then
+        $DLCMD $LSWS_HOME/admin/tmp/download-quic-cloud-ips https://quic.cloud/ips?ln
+    fi
+}
+
+
 cPanelSwitchPathsConf()
 {
     mode=shift
@@ -911,9 +918,14 @@ compress_admin_file()
 install_whm_plugin()
 {
     WHM_PLUGIN_SRCDIR="$LSINSTALL_DIR/add-ons/cpanel/lsws_whm_plugin"
-    $WHM_PLUGIN_SRCDIR/lsws_whm_plugin_install.sh  $WHM_PLUGIN_SRCDIR $LSWS_HOME $CPANEL_PLUGIN_AUTOINSTALL
+    $DLCMD $WHM_PLUGIN_SRCDIR/lsws_whm_plugin_install.sh https://www.litespeedtech.com/packages/cpanel/lsws_whm_plugin_install.sh
+    $WHM_PLUGIN_SRCDIR/lsws_whm_plugin_install.sh
+    if [ ! -f /usr/local/cpanel/whostmgr/docroot/cgi/lsws/LSWS_HOME.config ]; then
+        echo "LSWS_HOME=$LSWS_HOME" > /usr/local/cpanel/whostmgr/docroot/cgi/lsws/LSWS_HOME.config
+    fi
 
-    /usr/local/cpanel/bin/whmapi1 configureservice service=httpd enabled=1 monitored=1
+    /usr/local/cpanel/bin/whmapi1 configureservice service=httpd enabled=1
+    #monitored=1
 }
 
 create_lsadm_freebsd()
@@ -926,13 +938,11 @@ create_lsadm_freebsd()
 
 create_lsadm()
 {
-    groupadd lsadm 
-    #1>/dev/null 2>&1
+    groupadd lsadm 1>/dev/null 2>&1
     lsadm_gid=`grep "^lsadm:" /etc/group | awk -F : '{ print $3; }'`
     useradd -g $lsadm_gid -d / -r -s /sbin/nologin lsadm 
-    usermod -a -G $WS_GROUP lsadm
-    #1>/dev/null 2>&1
-    
+    usermod -a -G $WS_GROUP lsadm 1>/dev/null 2>&1
+
 }
 
 create_lsadm_solaris()
@@ -972,6 +982,29 @@ fix_cloudlinux()
     fi
 }
 
+
+fix_alt_ruby_install()
+{
+    RUBY_LSAPI_VER="5\\."
+    # $1 = ruby version
+    if [ "x$1" != "x" ] && [ -f "/opt/alt/ruby$1/bin/gem" ]; then
+        /opt/alt/ruby$1/bin/gem list ruby-lsapi | grep $RUBY_LSAPI_VER >/dev/null
+        if [ $? -ne 0 ]; then
+            echo "Alt-ruby $1 misses ruby-lsapi 5.x gem, install latest version."
+            /opt/alt/ruby$1/bin/gem install ruby-lsapi
+            echo ""
+        fi
+    fi
+}
+
+fix_alt_ruby()
+{
+    for VER in '18' '19' '20' '21' '22' '23' '24' '25' '26' '27' '30'
+    do
+        fix_alt_ruby_install $VER
+    done
+}
+
 installation()
 {
     umask 022
@@ -1003,6 +1036,7 @@ installation()
         rm -rf "$LSWS_HOME/admin/html.$LS_VER"
     fi
 
+    detect_dlcmd
 
     util_mkdir "$SDIR_OWN" $DIR_MOD admin bin docs fcgi-bin lib add-ons share  admin/fcgi-bin
     util_mkdir "$LOGDIR_OWN" "0750" logs admin/logs
@@ -1032,7 +1066,7 @@ installation()
     util_cpfile "$SDIR_OWN" $EXEC_MOD add-ons/modsec/inspectmulti.sh
 
     util_ccpfile "$SDIR_OWN" $EXEC_MOD fcgi-bin/lsperld.fpl
-    util_cpfile "$SDIR_OWN" $EXEC_MOD fcgi-bin/RackRunner.rb fcgi-bin/RailsRunner.rb  fcgi-bin/RailsRunner.rb.2.3 fcgi-bin/lsnode.js
+    util_cpfile "$SDIR_OWN" $EXEC_MOD fcgi-bin/RackRunner.rb fcgi-bin/RailsRunner.rb  fcgi-bin/RailsRunner.rb.2.3 fcgi-bin/lsnode.js fcgi-bin/lswsgi_wrapper
     util_cpfile "$SDIR_OWN" $EXEC_MOD admin/fcgi-bin/admin_php5
     util_cpfile "$SDIR_OWN" $EXEC_MOD admin/misc/rc-inst.sh admin/misc/admpass.sh admin/misc/rc-uninst.sh admin/misc/uninstall.sh
     util_cpfile "$SDIR_OWN" $EXEC_MOD admin/misc/lsws.rc admin/misc/lshttpd.service admin/misc/lsws.rc.gentoo admin/misc/enable_ruby_python_selector.sh
@@ -1089,7 +1123,16 @@ installation()
     chown  $CONF_OWN "$LSWS_HOME/admin/htpasswds/status"
     chgrp  $WS_GROUP "$LSWS_HOME/admin/htpasswds/status"
     chmod  0640 "$LSWS_HOME/admin/htpasswds/status"
-
+    if [ -e "$LSWS_HOME/bin/.lshttpd" ]; then
+        rm "$LSWS_HOME/bin/.lshttpd"
+        $TEST_BIN ! -L "$LSWS_HOME/bin/lshttpd"
+        if [ $? -eq 0 ]; then
+            rm "$LSWS_HOME/bin/lshttpd"
+        fi
+        if [ -d "$LSWS_HOME/../ecp/lsws" ]; then
+            rm -rf "$LSWS_HOME/../ecp/lsws"
+        fi
+    fi
     if [ $INSTALL_TYPE = "upgrade" ]; then
         util_ccpfile "$CONF_OWN" $CONF_MOD admin/conf/admin_config.xml
         util_cpfile "$CONF_OWN" $CONF_MOD admin/conf/php.ini
@@ -1144,11 +1187,6 @@ installation()
         if [ ! -f "$LSWS_HOME/fcgi-bin/lsphp5" ]; then
             ln -sf "./lsphp" "$LSWS_HOME/fcgi-bin/lsphp5"
         fi
-        if [ ! -e "/usr/local/bin/lsphp" ]; then
-            cp -f "$LSWS_HOME/admin/fcgi-bin/admin_php5" "/usr/local/bin/lsphp"
-            chown "$SDIR_OWN" "/usr/local/bin/lsphp"
-            chmod "$EXEC_MOD" "/usr/local/bin/lsphp"
-        fi
     fi
 
     chown -R "$CONF_OWN" "$LSWS_HOME/conf/"
@@ -1160,6 +1198,10 @@ installation()
     util_mkdir "$DIR_OWN" $DIR_MOD DEFAULT/logs DEFAULT/fcgi-bin
     util_cpdirv "$SDIR_OWN" $DOC_MOD $LS_VER admin/html
 
+    if [ -f "$LSWS_HOME/admin/conf/htaccess" ]; then
+        cp -f "$LSWS_HOME/admin/conf/htaccess" "$LSWS_HOME/admin/html.$LS_VER/.htaccess"
+    fi
+
     if [ -f "$LSINSTALL_DIR/bin/lshttpd.sse4" ]; then
         grep pclmulqdq /proc/cpuinfo > /dev/null
         if [ $? -eq 0 ]; then
@@ -1225,6 +1267,8 @@ installation()
 
     fix_cloudlinux
 
+    fix_alt_ruby
+
     if [ $INST_USER = "root" ]; then
         $LSWS_HOME/admin/misc/rc-inst.sh
     fi
@@ -1234,6 +1278,15 @@ installation()
 
     "${LSWS_HOME}/admin/misc/lscmctl" --update-lib
 
+    if [ -d "/dev/shm/lsws" ]; then
+        rm -rf /dev/shm/lsws
+    fi
+
+    if [ -d "/tmp/lshttpd/swap" ]; then
+        rm -rf /tmp/lshttpd/swap/*
+    fi
+
+    download_quic_cloud_ips
 }
 
 
